package example

import (
	"io"

	"zks.com/gotcp"
)

var (
	fullBuf *bytes.Buffer = bytes.NewBuffer([]byte{}) //缓存区
	SOH     []byte        = []byte{0xBE, 0xBE}        //开始标记 start of head
)

// Packet: SOH + Data
type StartTagPacketDelegate struct {
	pacLen  int32
	pacType int32
	pacData []byte
}

func (p *StartTagPacketDelegate) Serialize() []byte {
	buf := make([]byte, 8+len(p.pacData))
	copy(buf[0:4], gotcp.Int32ToBytes(p.pacLen))
	copy(buf[4:8], gotcp.Int32ToBytes(p.pacType))
	copy(buf[8:], p.pacData)
	return buf
}

func (p *StartTagPacketDelegate) GetLen() int32 {
	return p.pacLen
}

func (p *StartTagPacketDelegate) GetTypeInt() int32 {
	return p.pacType
}

func (p *StartTagPacketDelegate) GetTypeString() string {
	return ""
}

func (p *StartTagPacketDelegate) GetData() []byte {
	return p.pacData
}

func NewPacket(pacType int32, pacData []byte) *gotcp.Packet {
	packet := new(gotcp.Packet)
	packet.Delegate = &LtvPacketDelegate{
		pacLen:  int32(8) + int32(len(pacData)),
		pacType: pacType,
		pacData: pacData,
	}
	return packet
}

type StartTagProtocol struct {
}

func (this *StartTagProtocol) ReadPacket(r io.Reader, MaxPacketLength int32) (*gotcp.Packet, error) {
	var (
		pacBLen  []byte = make([]byte, 4)
		pacBType []byte = make([]byte, 4)
		pacLen   int32
	)

	// read pacLen
	if n, err := io.ReadFull(r, pacBLen); err != nil && n != 4 {
		return nil, gotcp.ReadPacketError
	}
	if pacLen = gotcp.BytesToInt32(pacBLen); pacLen > MaxPacketLength {
		return nil, gotcp.PacketTooLargeError
	}

	// read pacType
	if n, err := io.ReadFull(r, pacBType); err != nil && n != 4 {
		return nil, gotcp.ReadPacketError
	}

	// read pacData
	pacData := make([]byte, pacLen-8)
	if n, err := io.ReadFull(r, pacData); err != nil && n != int(pacLen) {
		return nil, gotcp.ReadPacketError
	}

	return NewPacket(gotcp.BytesToInt32(pacBType), pacData), nil
}
