package main

import (
	"bufio"
	"net"
	"os"
	"os/signal"
	"runtime"
	"syscall"
	"time"

	"github.com/nsqio/go-nsq"
	"zks.com/gotcp"
	service "zks.com/newbbfrontsocketv2/blackbox"
	utils "zks.com/socketutils"
	"zks.com/zUtils/redis"
)

var port string = "11100"

func main() {
	service.Config = service.ReadAppConfig("conf/app.conf")
	//初始化缓存区
	redis.InitRedisAuth(service.Config.Send.Redis.Poolnum, service.Config.Send.Redis.Addr, service.Config.Send.Redis.Pwd)
	//redis.InitRedis(service.Config.Send.Redis.Poolnum, service.Config.Send.Redis.Addr)
	//nsq
	cfg := nsq.NewConfig()
	service.Producernsq, _ = nsq.NewProducer(service.Config.Send.Nsq.Nsqdtcpaddr, cfg)
	service.Producernsqrequest, _ = nsq.NewProducer(service.Config.Send.Nsqrequest.Nsqdtcpaddr, cfg)
	initport()
	//初始化日志设定
	os.Mkdir("./runlog", os.ModePerm)
	utils.SetLevel(utils.LevelInformational)
	utils.SetLogger("file", `{"filename":"runlog/runlog.log"}`)
	//启动服务
	runtime.GOMAXPROCS(runtime.NumCPU())
	tcpAddr, err := net.ResolveTCPAddr("tcp4", ":"+port)

	utils.LogError(err)

	listener, err := net.ListenTCP("tcp", tcpAddr)
	utils.LogError(err)
	utils.Informational("服务启动，端口:", port)

	config := &gotcp.Config{
		AcceptTimeout:          10 * 60 * time.Second,
		ReadTimeout:            20 * 60 * time.Second,
		WriteTimeout:           120 * time.Second,
		MaxPacketLength:        20240,
		SendPacketChanLimit:    50,
		ReceivePacketChanLimit: 100,
	}

	//因未获取到此代码引用，故暂时使用其他方法
	//delegate := &mos.MosConnDelegate{}
	//protocol := &mos.MosProtocol{}
	delegate := &service.BBoxConnDelegate{}
	protocol := &service.BBoxProtocol{}
	svr := gotcp.NewServer(config, delegate, protocol)
	go svr.Start(listener)

	//停止服务
	ch := make(chan os.Signal)
	signal.Notify(ch, syscall.SIGINT, syscall.SIGTERM)
	utils.Informational("Signal:", <-ch)

	svr.Stop()
}
func initport() {
	f, err := os.OpenFile("conf/port.txt", os.O_RDONLY, 0)
	if err != nil {
		panic(err)
	}
	defer f.Close()

	br := bufio.NewReader(f)

	//每次读取一行
	port, _ = br.ReadString('\n')
}
